
(function($, owner) {
    owner.deviceId = null;
    owner.serviceId = null;
	owner.characteristicId = null;
	
	owner.pageSize = "58mm";
    
    /**
     * 打印纸一行最大的字节
     */
    var LINE_BYTE_SIZE = 32;
    var LINE_BYTE_SIZE_80 = 47;
    var LINE_BYTE_SIZE_110 = 64;
	var LINE_BYTE_SIZE_241 = 100;

    /**
     * 打印三列时，中间一列的中心线距离打印纸左侧的距离
     */
    var LEFT_LENGTH = 14;
    var LEFT_LENGTH_80 = 24;
    var LEFT_LENGTH_110 = 32;
	var LEFT_LENGTH_241 = 50;

    /**
     * 打印三列时，中间一列的中心线距离打印纸右侧的距离
     */
    var RIGHT_LENGTH = 14;
    var RIGHT_LENGTH_80 = 24;
    var RIGHT_LENGTH_110 = 32;
	var RIGHT_LENGTH_241 = 50;

    /**
     * 打印三列时，第一列汉字最多显示几个文字
     */
    var LEFT_TEXT_MAX_LENGTH = 5;
    var LEFT_TEXT_MAX_LENGTH_80 = 9;
    var LEFT_TEXT_MAX_LENGTH_110 = 11;
	var LEFT_TEXT_MAX_LENGTH_241 = 22;

    owner.init = function(deviceId, serviceId, characteristicId) {
        owner.deviceId = deviceId;
        owner.serviceId = serviceId;
        owner.characteristicId = characteristicId;
    }
	
	owner.setPageSize = function(pageSize){
		owner.pageSize = pageSize;
	}
    
    owner.getBytesLength = function(val) {
		if(val==null){
			val = '';
		}
		var len = 0;
		for (var i = 0; i < val.length; i++) {
			 var a = val.charAt(i);
			 if (a.match(/[^\x00-\xff]/ig) != null) 
			{
				len += 2;
			}
			else
			{
				len += 1;
			}
		}
		return len;
    };
       
    owner.getTwoCol = function(leftText, rightText) {
        var sb = leftText;
        
        var leftTextLength = owner.getBytesLength(leftText);
        var rightTextLength = owner.getBytesLength(rightText);
		
		var lineByteSize = LINE_BYTE_SIZE;
		if(owner.pageSize == '80mm'){
			lineByteSize = LINE_BYTE_SIZE_80;
		}else if(owner.pageSize == '110mm'){
			lineByteSize = LINE_BYTE_SIZE_110;
		}else if(owner.pageSize == '241mm'){
			lineByteSize = LINE_BYTE_SIZE_241;
		}

        // 计算两侧文字中间的空格
        var marginBetweenMiddleAndRight = parseInt(lineByteSize) - parseInt(leftTextLength) - parseInt(rightTextLength);

        for (var i = 0; i < marginBetweenMiddleAndRight; i++) {
            sb += (" ");
        }
        sb += (rightText);
        
        return sb;
    }
    
    owner.getThreeCol = function(leftText, middleText, rightText) {
    	
        var sb = leftText;
        
        var leftTextLength = owner.getBytesLength(leftText);
        var middleTextLength = owner.getBytesLength(middleText);
        var rightTextLength = owner.getBytesLength(rightText)+2;
		
		var leftLength = LEFT_LENGTH;
		var rightLength = RIGHT_LENGTH;
		if(owner.pageSize == '80mm'){
				leftLength = LEFT_LENGTH_80;
				rightLength = RIGHT_LENGTH_80;
		}else if(owner.pageSize == '110mm'){
				leftLength = LEFT_LENGTH_110;
				rightLength = RIGHT_LENGTH_110;
		}else if(owner.pageSize == '241mm'){
			leftLength = LEFT_LENGTH_241;
			rightLength = RIGHT_LENGTH_241;
		}
        
        //左侧文字
        if(leftTextLength<leftLength){
	    		for(var j=leftTextLength;j<leftLength;j++){
	    			sb += " ";
	    		}
	    	}else{
	    		sb += "\n                ";
	    	}
        
        //中间文字
        sb += (middleText);

        // 计算右侧文字和中间文字的空格长度
        var marginBetweenMiddleAndRight = parseInt(rightLength) - parseInt(middleTextLength / 2) - parseInt(rightTextLength);
        for (var i = 0; i < marginBetweenMiddleAndRight-1; i++) {
            sb += " ";
        }
        
        sb += rightText;

        return sb;
    }
	
	owner.getFourCol = function(leftText, middleText,middleText2, rightText) {
		
	    var sb = leftText;
	    
	    var leftTextLength = owner.getBytesLength(leftText);
	    var middleTextLength = owner.getBytesLength(middleText);
	    var middleTextLength2 = owner.getBytesLength(middleText2);
	    var rightTextLength = owner.getBytesLength(rightText);
	    
	    var lineByteSize = LINE_BYTE_SIZE;
	    if(owner.pageSize == '80mm'){
	    		lineByteSize = LINE_BYTE_SIZE_80;
	    }else if(owner.pageSize == '110mm'){
	    		lineByteSize = LINE_BYTE_SIZE_110;
	    }else if(owner.pageSize == '241mm'){
	    		lineByteSize = LINE_BYTE_SIZE_241;
	    }
	    
	    var avg = Number(lineByteSize)/4;
	    
	    //左侧文字
	    var flag = false; //标记是否换行
	    if(leftTextLength<avg){
	    		for(var j=leftTextLength;j<avg;j++){
	    			sb += " ";
	    		}
	    	}else{
	    		flag = true;
	    		sb += "\n";
	    		for(var j=0;j<avg;j++){
	    			sb += " ";
	    		}
	    	}
	    
	    //中间文字
	    sb += (middleText);
	
	    //计算第三列的打印位置
	    var length = owner.getBytesLength(sb);
	    if(!flag){
	        if(length < Number(avg)*2){
		        for (var i = 0; i < (Number(avg)*2-length); i++) {
		            sb += " ";
		        }
		    }
	    }else{
	    		var marginLength =  avg - middleTextLength - 3;
	    		for (var i = 0; i < (marginLength); i++) {
		        sb += " ";
		    }
	    }
	    sb += middleText2;
	    
	    length = owner.getBytesLength(sb);
	    if(!flag){
	        if(length<Number(avg)*3){
		        for (var i = 0; i < (Number(avg)*3-length); i++) {
		            sb += " ";
		        }
		    }
	    }else{
	    		var marginLength =  avg - middleTextLength2 - 3;
	    		for (var i = 0; i < (marginLength); i++) {
		            sb += " ";
		    }
	    }
		sb += rightText;
		
	    return sb;
	}
	
	/**
	 * @param {Object} text
	 * @param {Object} n 字体放大倍数
	 */
	owner.buildCenterText = function(text, n){
		var sb = "";
        
		var textLength = owner.getBytesLength(text);
        var m = (LINE_BYTE_SIZE_241/n - textLength) / 2;
        var n = (LINE_BYTE_SIZE_241/n - textLength) % 2;

		for (var j = 0; j < m; j++) {
			sb += (" ");
		}
		sb += (text);
		
		for (var j = 0; j < m; j++) {
			sb += (" ");
		}
		if (n == 1) {
           sb += (" ");
        }
		
        return sb;
	}
	
	//左右两侧填充空格
	owner.fillBlankSpace = function(text, n){
		var sb = "";
		
		for (var j = 0; j < n; j++) {
			sb += (" ");
		}
		
		sb += (text);
		
		for (var j = 0; j < n; j++) {
			sb += (" ");
		}
		
        return sb;
	}
	
	//右侧填充空格
	owner.fillRightBlankSpace = function(text, n){
		var sb = text;
		for (var j = 0; j < n; j++) {
			sb += (" ");
		}
		
        return sb;
	}
	
	//左侧填充空格
	owner.fillLeftBlankSpace = function(text, n){
		var sb = "";
		for (var j = 0; j < n; j++) {
			sb += (" ");
		}
		
        return sb + text;
	}

    // 设置字体大小
    owner.getFontSizeCmd = function(n) {
        var font = [0x1D, 0X21, n]
        return font;
    };
	
	//设定汉字4倍角打印模式
    owner.getDoubleCmd = function(n) {
        var font = [0x1C, 0x57, 0x01]
        return font;
    };
	
	//设定取消汉字4倍角打印模式
    owner.getDoubleCancelCmd = function(n) {
        var font = [0x1C, 0x57, 0x00]
        return font;
    };
    
     // 重置打印机
    owner.getResetCmd = function() {
        var reset = [0x1B, 0X40];
        return reset;
    };

    owner.getAlignCenterCmd = function(){
    		var reset = [0x1b, 0x61, 0x01];
		return reset;
    };
	
	owner.getRowNumCmd = function(){
    		var reset = [27, 67, 33];
		return reset;
    };
    
    owner.getAlignLeftCmd = function(){
    	var reset = [0x1b, 0x61, 0x00];
		return reset;
    };
    
    owner.getAlignRightCmd = function(){
    	var reset = [0x1b, 0x61, 0x02];
		return reset;
    };
    
    // 结束打印
    owner.getEndCmd = function() {
        var end = [0x0C];
        return end;
    };

    // 结束打印
    owner.getNextPageCmd = function() {
        var end = [0x1B, 0x4E , 0x03];
        return end;
    };
	
    // 切纸（暂不可用）
    owner.getCutPageCmd = function() {
        // 发送切纸指令  
        var end = [0x1B, 0x69];
        return end;
    };
	
	owner.print = function(buffer){
		return new Promise(function (resolve, reject) {
				
			plus.bluetooth.writeBLECharacteristicValue({
				deviceId: owner.deviceId,
				serviceId: owner.serviceId,
				characteristicId: owner.characteristicId,
				value: buffer,
				success: function(e) {
					console.log('写入数据成功!');
					resolve(true);
				},
				fail: function(e) {
					console.log('写入数据失败! ' + JSON.stringify(e));
					reject(false);
				}
			});
		});
	}
	
	owner.printList = function(array){

		var promise = new Promise(function(resolve, reject) {
		
			var promiseArray = [];
			for(var i=0;i<array.length;i++){
				var s = array[i];
				promiseArray.push(owner.print(s));
			}

			Promise.all(promiseArray).then(arr => {
				resolve(true);
			});
			
		});
		
		return promise;
	}
	
	owner.closeBLEConnection = function(){
		return new Promise(function (resolve, reject) {
			plus.bluetooth.closeBLEConnection({
				deviceId:owner.deviceId,
				success:function(e){
					console.log('close success: '+JSON.stringify(e));
					resolve(true);
				},
				fail:function(e){
					console.log('close failed: '+JSON.stringify(e));
					reject(false);
				}
			});
		});
	}
	
	owner.closeBluetoothAdapter = function(){
		plus.bluetooth.stopBluetoothDevicesDiscovery({
			success:function(e){
				console.log('stop discovery success: '+JSON.stringify(e));
				plus.bluetooth.closeBluetoothAdapter({
					success:function(e){
						console.log('close success: '+JSON.stringify(e));
					},
					fail:function(e){
						console.log('close failed: '+JSON.stringify(e));
					}
				});
			},
			fail:function(e){
				console.log('stop discovery failed: '+JSON.stringify(e));
			}
		});
	}
	
	// 打印字符串
	owner.getMark = function(mark) {
		var string = '';
		
		var lineByteSize = LINE_BYTE_SIZE;
		if(owner.pageSize == '80mm'){
			lineByteSize = LINE_BYTE_SIZE_80;
		}else if(owner.pageSize == '110mm'){
				lineByteSize = LINE_BYTE_SIZE_110;
		}else if(owner.pageSize == '241mm'){
				lineByteSize = LINE_BYTE_SIZE_241;
		}
		for(var i=0;i<lineByteSize;i++){
			string += mark;
		}
	    return string;
	};

}(mui, window.PrintUtil4Ios = {}));

// 连接蓝牙设备
function createConnection(deviceId){
	return new Promise(function (resolve, reject) {
		plus.bluetooth.createBLEConnection({
			deviceId:deviceId,
			success:function(e){
				console.log('create connection success: '+JSON.stringify(e));		
				
				resolve(true);
			},
			fail:function(e){
				console.log('create connection failed: '+JSON.stringify(e));
				
				reject(false);
			}
		});
	});
}

// 获取蓝牙设备的所有服务
function getServices(deviceId){
	return new Promise(function (resolve, reject) {
		setTimeout(function(){
			plus.bluetooth.getBLEDeviceServices({
				deviceId:deviceId,
				success:function(e){
					console.log('get service success: ');
					var services = e.services;
					resolve(JSON.stringify(services));
				},
				fail:function(e){
					reject([]);
				}
			});
		},600);
		
	});
}

 function createPromise(deviceId,serviceId) {
	 
	var promise = new Promise(function(resolve, reject) {
		
		plus.bluetooth.getBLEDeviceCharacteristics({
			deviceId:deviceId,
			serviceId:serviceId,
			success:function(e){
				console.log('get Characteristics success: ');
				var cs = e.characteristics;
				for(var i=0;i<cs.length;i++){
					cs[i].serviceId = serviceId;
				}
				resolve(cs);
			},
			fail:function(e){
				reject([]);
			}
		});
		
	});
	
	return promise;
}


// 获取蓝牙设备指定服务中所有特征值
function getCharacteristics(deviceId,services){
	
	var promise = new Promise(function(resolve, reject) {
		setTimeout(function(){
			var promiseArray = [];
			
			var ss = eval(services);
			
			for(var i=0;i<ss.length;i++){
				var s = ss[i];
				promiseArray.push(createPromise(deviceId,s.uuid));
			}

			Promise.all(promiseArray).then(arr => {
				var cs = [];
				
				for(var i=0;i<arr.length;i++){
					var item = arr[i];
					
					for(var j=0;j<item.length;j++){
						
						cs.push(item[j]);
					}
				}

				resolve({deviceId:deviceId , characteristics:cs});
			});
		},500);
	});
	
	return promise;
}


// 断开蓝牙设备
function closeConnection(deviceId){
	plus.bluetooth.closeBLEConnection({
		deviceId:deviceId,
		success:function(e){
			console.log('close success: '+JSON.stringify(e));
		},
		fail:function(e){
			console.log('close failed: '+JSON.stringify(e));
		}
	});
}

//开始搜索蓝牙
function startBluetoothDiscovery(){
	
	//1、初始化蓝牙模块
	plus.bluetooth.openBluetoothAdapter({
		success:function(e){
			console.log('1-开始：打开蓝牙模块成功: '+JSON.stringify(e));
			
			//2、开始搜索附近的蓝牙设备
			plus.bluetooth.startBluetoothDevicesDiscovery({
				success:function(e){
					console.log('开始搜索成功: '+JSON.stringify(e));
					
				},
				fail:function(e){
					console.log('开始搜索失败: '+JSON.stringify(e));
					
				}
			});
			
		},
		fail:function(e){
			console.log('1-结束：打开蓝牙模块失败: '+JSON.stringify(e));
		}
	});
	
}


// 停止搜索蓝牙
function stopBluetoothDiscovery(){
	
	//停止搜寻附近的蓝牙外围设备
	plus.bluetooth.stopBluetoothDevicesDiscovery({
		success:function(e){
			console.log('停止成功: '+JSON.stringify(e));
			
			//关闭蓝牙模块成功
			plus.bluetooth.closeBluetoothAdapter({
				success:function(e){
					console.log('关闭蓝牙模块成功: '+JSON.stringify(e));
					
					
				},
				fail:function(e){
					console.log('关闭蓝牙模块失败: '+JSON.stringify(e));
					
				}
			});
			
		},
		fail:function(e){
			console.log('停止失败: '+JSON.stringify(e));
			
		}
	});
	
}
